package io.gitlab.jfronny.inceptum.cli;

import io.gitlab.jfronny.inceptum.common.MetaHolder;
import io.gitlab.jfronny.inceptum.common.Utils;
import io.gitlab.jfronny.inceptum.launcher.system.instance.Instance;
import io.gitlab.jfronny.inceptum.launcher.system.instance.InstanceList;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.List;

public abstract class BaseInstanceCommand extends Command {
    protected BaseInstanceCommand(String help, String usage, List<String> aliases, List<Command> subCommands) {
        super(help, mutateUsage(usage), aliases, subCommands);
    }

    private static String mutateUsage(String usage) {
        StringBuilder sb = new StringBuilder();
        for (String s : usage.split("\n")) {
            if (s.isBlank())
                sb.append("\n<instance>");
            else
                sb.append("\n<instance> ").append(s);
        }
        return sb.substring(1);
    }

    @Override
    protected void invoke(CommandArgs args) {
        if (args.length == 0) {
            Utils.LOGGER.error("You must specify an instance to commit in");
            return;
        }
        Path instancePath = MetaHolder.INSTANCE_DIR.resolve(args[0]);
        if (!Files.exists(instancePath)) {
            Utils.LOGGER.error("Invalid instance: \"" + args[0] + "\"");
            return;
        }
        Instance instance;
        try {
            instance = InstanceList.read(instancePath);
        } catch (IOException e) {
            Utils.LOGGER.error("Could not read instance metadata", e);
            return;
        }
        try {
            invoke(args.subArgs(), instance);
        } catch (Exception e) {
            Utils.LOGGER.error("Could not execute command", e);
            return;
        }
    }

    protected abstract void invoke(CommandArgs args, Instance instance) throws Exception;
}
